==> emma-flink-0.2.1-sources.jar: org/emmalanguage/compiler/FlinkMacro.scala <==
/*
 * Copyright © 2014 TU Berlin (emma@dima.tu-berlin.de)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.emmalanguage
package compiler

import com.typesafe.config.Config

import scala.reflect.macros.blackbox

class FlinkMacro(val c: blackbox.Context) extends MacroCompiler with FlinkCompiler {

  import UniverseImplicits._

  override protected val baseConfig = "reference.emma.onFlink.conf" +: super.baseConfig

  def onFlinkImpl1[T](e: c.Expr[T]): c.Expr[T] = {
    val cfg = loadConfig(configPaths())
    val res = pipeline(cfg)(e)
    val pre = memoizedTypeInfos(res)
    if (cfg.getBoolean("emma.compiler.print-result")) {
      c.warning(e.tree.pos, api.Tree.show(res))
    }
    c.Expr[T](unTypeCheck(res match {
      case api.Block(stats, expr) => api.Block(pre ++ stats, expr)
      case _ => api.Block(pre, res)
    }))
  }

  def onFlinkImpl2[T](config: c.Expr[String])(e: c.Expr[T]): c.Expr[T] = {
    val cfg = loadConfig(configPaths(Some(config.tree)))
    val res = pipeline(cfg)(e)
    val pre = memoizedTypeInfos(res)
    if (cfg.getBoolean("emma.compiler.print-result")) {
      c.warning(e.tree.pos, api.Tree.show(res))
    }
    c.Expr[T](unTypeCheck(res match {
      case api.Block(stats, expr) => api.Block(pre ++ stats, expr)
      case _ => api.Block(pre, res)
    }))
  }

  private def pipeline(cfg: Config): c.Expr[Any] => u.Tree = {
    val xfms = Seq.newBuilder[u.Tree => u.Tree]
    // standard prefix
    xfms ++= Seq(
      Lib.expand,
      Core.lift
    )
    // optional optimizing rewrites
    if (cfg.getBoolean("emma.compiler.opt.cse")) {
      xfms += Core.cse
    }
    if (cfg.getBoolean("emma.compiler.opt.fold-fusion")) {
      xfms += Optimizations.foldFusion
    }
    if (cfg.getBoolean("emma.compiler.opt.auto-cache")) {
      xfms += Backend.addCacheCalls
    }

    xfms += Comprehension.combine

    xfms += Backend.specialize(FlinkAPI)

    cfg.getString("emma.compiler.lower") match {
      case "trampoline" =>
        xfms += Core.trampoline
      case "dscfInv" =>
        xfms += Core.dscfInv
    }

    xfms += removeShadowedThis

    // construct the compilation pipeline
    pipeline()(xfms.result(): _*).compose(_.tree)
  }
}

==> emma-flink-0.2.2-sources.jar: org/emmalanguage/compiler/FlinkMacro.scala <==
/*
 * Copyright © 2014 TU Berlin (emma@dima.tu-berlin.de)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.emmalanguage
package compiler

import com.typesafe.config.Config

import scala.reflect.macros.blackbox

class FlinkMacro(val c: blackbox.Context) extends MacroCompiler with FlinkCompiler {

  def onFlinkImpl1[T](e: c.Expr[T]): c.Expr[T] =
    onFlink(loadConfig(configPaths()))(e)

  def onFlinkImpl2[T](config: c.Expr[String])(e: c.Expr[T]): c.Expr[T] =
    onFlink(loadConfig(configPaths(Some(config.tree))))(e)

  def onFlink[T](cfg: Config)(e: c.Expr[T]): c.Expr[T] = {
    // construct the compilation pipeline
    val xfms = transformations(cfg)
    // apply the pipeline to the input tree
    val rslt = pipeline()(xfms: _*)(e.tree)
    // optionally, print the result
    if (cfg.getBoolean("emma.compiler.print-result")) {
      c.warning(e.tree.pos, api.Tree.show(rslt))
    }
    // wrap the result in an Expr and return it
    c.Expr[T](unTypeCheck(rslt))
  }
}
